#!/usr/bin/env python


import unittest


from .left_right import LeftRight


class BasicObjectForTest(object):
    def __init__(self, val1, avg, _max):
        self.val1 = val1
        self.avg = avg
        self.max = _max

    def __eq__(self, other):
        return (self.val1, self.avg, self.max) == (other.val1, other.avg, other.max)


class ObjectWithAvgForTest(object):
    def __init__(self, val):
        self.val = val

    @staticmethod
    def avg(left, right):
        return ObjectWithAvgForTest(left.val + right.val)

    def __eq__(self, other):
        return self.val == other.val


class LeftRightTest(unittest.TestCase):
    def test_construction(self):
        LeftRight(1, 2)
        LeftRight(1.0, 2.0)
        LeftRight('left', 'right')
        LeftRight({'a': 1}, {'a': 2})

        LeftRight(1, None)
        LeftRight(None, 1)
        LeftRight(None, None)

        with self.assertRaises(ValueError):
            LeftRight(1, 1.0)

    def test_avg(self):
        self.assertEqual(LeftRight(1, 1).avg(), 1)
        self.assertEqual(LeftRight(None, 1).avg(), 1)
        self.assertEqual(LeftRight(1, None).avg(), 1)

        self.assertEqual(LeftRight(1, 3).avg(), 2)
        self.assertEqual(LeftRight(1, 2).avg(), 1)

        self.assertEqual(LeftRight(1.0, 3.0).avg(), 2.0)
        self.assertEqual(LeftRight(1.0, 2.0).avg(), 1.5)

        left = BasicObjectForTest(1, 2, 3)
        right = BasicObjectForTest(1, 2, 3)
        self.assertEqual(LeftRight(left, right).avg(), BasicObjectForTest(1, 2, 3))
        same_object = BasicObjectForTest(1, 2, 3)
        left = BasicObjectForTest(1.0, 1, same_object)
        right = BasicObjectForTest(2.0, 3, same_object)
        self.assertEqual(LeftRight(left, right).avg(), BasicObjectForTest(1.5, 2, same_object))

        left = ObjectWithAvgForTest(1)
        right = ObjectWithAvgForTest(2)
        self.assertEqual(LeftRight(left, right).avg(), ObjectWithAvgForTest(3))

    def test_equality(self):
        lr = LeftRight(1, 1)
        self.assertEqual(lr, lr)
        self.assertEqual(LeftRight(1, 1), LeftRight(1, 1))

        self.assertNotEqual(LeftRight(1, 1), LeftRight(1, 2))
        self.assertNotEqual(LeftRight(1, 1), LeftRight(2, 1))
        self.assertNotEqual(LeftRight(1, 1), LeftRight(2, 2))
        self.assertNotEqual(LeftRight(1, 1), None)

        with self.assertRaises(NotImplementedError):
            LeftRight(1, 1).__eq__('test')


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import sys


from .age import AgeCategory, EstimatedAge
from .context import BodyPosition, CompassBearing, Context, Present
from .individual import AgeSexStature, BurialInfo, Individual, LongBoneMeasurement, OsteologicalSex
from .joints import JointCondition, Joints
from .left_right import LeftRight
from .mouth import Mouth, Tooth
from .occupational_markers import EnthesialMarker, OccupationalMarkers
from .sex import Sex
from .trauma import Trauma, TraumaCategory


if sys.version_info < (3, 7):
    print('ERROR: Only Python 3.7 and above is supported')
    sys.exit(-1)


__title__ = 'bioarch'
__version__ = '0.0.32'
__author__ = 'Guy Taylor'

__all__ = ['AgeCategory', 'EstimatedAge'] + ['BodyPosition', 'CompassBearing', 'Context', 'Present'] + \
          ['AgeSexStature', 'BurialInfo', 'Individual', 'LongBoneMeasurement', 'OsteologicalSex'] + \
          ['JointCondition', 'Joints'] + ['EnthesialMarker', 'OccupationalMarkers'] + ['LeftRight'] + \
          ['Mouth', 'Tooth'] + ['Sex'] + ['Trauma', 'TraumaCategory']



#!/usr/bin/env python


import unittest


from pandas.api.types import CategoricalDtype


from .context import BodyPosition, CompassBearing, Context, Present


class BodyPositionTest(unittest.TestCase):
    def test_parse(self):
        self.assertEqual(BodyPosition.parse('SUPINE'), BodyPosition.SUPINE)
        self.assertEqual(BodyPosition.parse('supine'), BodyPosition.SUPINE)

        self.assertEqual(BodyPosition.parse(0), BodyPosition.SUPINE)
        self.assertEqual(BodyPosition.parse(1), BodyPosition.SUPINE_FIXED_LEGS)
        self.assertEqual(BodyPosition.parse(2), BodyPosition.CROUCHED)
        self.assertEqual(BodyPosition.parse(3), BodyPosition.CROUCHED_LEFT_SIDE)
        self.assertEqual(BodyPosition.parse(4), BodyPosition.CROUCHED_RIGHT_SIDE)
        self.assertEqual(BodyPosition.parse(5), BodyPosition.STOMACH)

        self.assertEqual(BodyPosition.parse('NA'), None)
        self.assertEqual(BodyPosition.parse(None), None)


class CompassBearingTest(unittest.TestCase):
    def test_order(self):
        self.assertEqual(sorted([CompassBearing.SOUTH, CompassBearing.NORTH]), [CompassBearing.NORTH, CompassBearing.SOUTH])
        # None is sorted first
        self.assertEqual(sorted([CompassBearing.SOUTH, CompassBearing.NORTH, None]), [None, CompassBearing.NORTH, CompassBearing.SOUTH])
        # Numbers are sorted as if they where CompassBearing.value
        self.assertEqual(sorted([0, 3, CompassBearing.EAST]), [0, CompassBearing.EAST, 3])
        # Invalid numbers fail
        with self.assertRaises(ValueError):
            [CompassBearing.SOUTH, -1].sort()

    def test_to_short_code(self):
        self.assertEqual(CompassBearing.NORTH.to_short_code(), 'N')
        self.assertEqual(CompassBearing.NORTH_EAST.to_short_code(), 'NE')
        self.assertEqual(CompassBearing.SOUTH.to_short_code(), 'S')

    def test_parse(self):
        self.assertEqual(CompassBearing.parse('NORTH'), CompassBearing.NORTH)
        self.assertEqual(CompassBearing.parse('N'), CompassBearing.NORTH)
        self.assertEqual(CompassBearing.parse(None), None)


class ContextTest(unittest.TestCase):
    def test_constructor_position(self):
        self.assertEqual(Context(BodyPosition.SUPINE, None, {}).body_position, BodyPosition.SUPINE, None)
        self.assertEqual(Context(None, None, {}).body_position, None)

        with self.assertRaises(ValueError):
            Context('SUPINE', None, {})

    def test_constructor_orientation(self):
        self.assertEqual(Context(None, CompassBearing.NORTH, {}).body_orientation, CompassBearing.NORTH)
        self.assertEqual(Context(None, None, {}).body_orientation, None)

        with self.assertRaises(ValueError):
            Context(None, 'NORTH', {})
        with self.assertRaises(ValueError):
            Context(None, 1, {})

    def test_constructor_tags(self):
        self.assertEqual(Context(None, None, {'thing': True}).tags['thing'], Present.PRESENT)
        self.assertEqual(Context(None, None, {'thing': False}).tags['thing'], Present.NOT_PRESENT)

        self.assertEqual(Context(None, None, {'thing': None}).tags['thing'], None)
        self.assertEqual(Context(None, None, {'thing': 'NA'}).tags['thing'], None)
        self.assertEqual(Context(None, None, {'thing': 'na'}).tags['thing'], None)

        with self.assertRaises(ValueError):
            Context(None, None, {'spear': 'foo'})

        with self.assertRaises(ValueError):
            Context(None, None, {'spear': 10})

    def test_to_pd_data_frame(self):
        context = Context(BodyPosition.SUPINE,
                          CompassBearing.NORTH,
                          {'spear': True,
                           'comb': 'NA',
                           'knife': None})

        df = context.to_pd_data_frame('id1')

        self.assertIsInstance(df['body_position_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['body_orientation_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['all_spear_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['weapons_cat'].dtypes, CategoricalDtype)
        self.assertEqual(df.to_json(orient='records'), '[{"body_position_cat":"SUPINE","body_position_val":0,"body_orientation_cat":"NORTH","body_orientation_val":0,"all_spear_cat":"PRESENT","all_spear_val":1,"all_comb_cat":null,"all_comb_val":null,"all_knife_cat":null,"all_knife_val":null,"utilitarian_cat":null,"utilitarian_val":null,"textile_cat":null,"textile_val":null,"equestrian_cat":null,"equestrian_val":null,"economic_cat":null,"economic_val":null,"organic_material_cat":null,"organic_material_val":null,"appearance_cat":null,"appearance_val":null,"burial_container_cat":null,"burial_container_val":null,"weapons_cat":"PRESENT","weapons_val":1,"iron_fragment_cat":null,"iron_fragment_val":null,"miscellaneous_cat":null,"miscellaneous_val":null}]')

        context = Context(None,
                          None,
                          {'spear': True,
                           'comb': 'NA',
                           'knife': None})

        df = context.to_pd_data_frame('id1')

        self.assertIsInstance(df['body_position_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['body_orientation_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['all_comb_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['appearance_cat'].dtypes, CategoricalDtype)
        self.assertEqual(df.to_json(orient='records'), '[{"body_position_cat":null,"body_position_val":null,"body_orientation_cat":null,"body_orientation_val":null,"all_spear_cat":"PRESENT","all_spear_val":1,"all_comb_cat":null,"all_comb_val":null,"all_knife_cat":null,"all_knife_val":null,"utilitarian_cat":null,"utilitarian_val":null,"textile_cat":null,"textile_val":null,"equestrian_cat":null,"equestrian_val":null,"economic_cat":null,"economic_val":null,"organic_material_cat":null,"organic_material_val":null,"appearance_cat":null,"appearance_val":null,"burial_container_cat":null,"burial_container_val":null,"weapons_cat":"PRESENT","weapons_val":1,"iron_fragment_cat":null,"iron_fragment_val":null,"miscellaneous_cat":null,"miscellaneous_val":null}]')

    def test_known_context_to_group(self):
        known_context_keys = {'knife': set(['utilitarian']),  # Should not be a weapons
                              'Whetstone': set(['utilitarian']),
                              'Awl': set(['utilitarian']),
                              'SCISSORS': set(['utilitarian']),
                              'iron_object': set(['iron_fragment']),
                              'textile': set(['textile']),
                              'spindle_whorl': set(['textile']),
                              'brooch': set(['appearance']),
                              'bracelet': set(['appearance']),
                              'lock': set(['miscellaneous']),
                              'keys': set(['miscellaneous']),
                              'Sword': set(['weapons']),
                              'axe': set(['weapons']),
                              'Iron_pole': set(['miscellaneous']),  # Should not be iron_fragment or weapons
                              'spear': set(['weapons']),
                              'shield_boss': set(['weapons']),
                              'cloak_fastener': set(['appearance']),
                              'iron_ring': set(['appearance']),  # SHould not be iron_fragment
                              'iron_nail/rivets': set(['iron_fragment']),
                              'transport': set(),
                              'Horse_equipment': set(['equestrian']),
                              'coffin_substitute': set(['burial_container']),
                              'coffin': set(['burial_container']),
                              'Mounts': set(['appearance']),
                              'thors_hammer': set(['miscellaneous']),
                              'beads': set(['appearance']),
                              'unidentified_bronze': set(['miscellaneous']),
                              'Bronze_disk': set(['miscellaneous']),
                              'SCALES': set(['economic']),
                              'coins': set(['economic']),
                              'VESSEL': set(['utilitarian']),
                              'comb': set(['appearance']),
                              'buckle': set(['appearance']),
                              'pot_sherd': set(['utilitarian']),
                              'FLINT_FLAKES': set(['utilitarian']),
                              'charcoal': set(['organic_material']),
                              'burned_bones': set(['organic_material']),
                              'quartz': set(['miscellaneous']),
                              'shell': set(['organic_material']),
                              'needle': set(['textile']),
                              'Flower': set(['organic_material']),
                              'animal_remains': set(['organic_material']),
                              'Body_Position': set(),
                              'disturbed': set(),
                              'decapitation': set(),
                              'double_grave': set(),
                              'Body_orientation': set(),
                              'Stone_layer': set()}
        for known_key, known_groups in known_context_keys.items():
            actual_groups = Context.group(known_key)
            self.assertEqual(actual_groups, known_groups, msg=f'{known_key} -> {actual_groups} != {known_groups}')


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import inspect
from statistics import mean
from typing import Any, cast, Generic, List, Optional, TypeVar


T = TypeVar('T')


def best_effort_avg(left: Optional[T], right: Optional[T]) -> Optional[T]:
    if left is None:
        return right
    if right is None:
        return left

    if isinstance(left, int):
        if not isinstance(right, int):
            raise TypeError
        return cast(T, int(mean((left, right))))

    if isinstance(left, float):
        if not isinstance(right, float):
            raise TypeError
        return cast(T, mean((left, right)))

    if right.__class__ != left.__class__:
        raise TypeError
    type_class = left.__class__

    if hasattr(type_class, 'avg') and inspect.isfunction(getattr(type_class, 'avg')):
        avg_func = getattr(type_class, 'avg')
        arg_spec = inspect.getfullargspec(avg_func)
        if len(arg_spec.args) != 2:
            raise ValueError
        if arg_spec.args[0] == 'self':
            raise ValueError
        if arg_spec.varargs is not None:
            raise ValueError
        if arg_spec.varkw is not None:
            raise ValueError
        return cast(T, avg_func(left, right))

    arg_spec = inspect.getfullargspec(type_class)
    if arg_spec.varargs is not None:
        raise ValueError
    if arg_spec.varkw is not None:
        raise ValueError

    avg_args: List[Any] = []
    for key in arg_spec.args[1:]:
        if key.startswith('_'):
            key = key[1:]
        left_val = getattr(left, key)
        right_val = getattr(right, key)
        avg = best_effort_avg(left_val, right_val)
        avg_args.append(avg)
    return type_class(*avg_args)  # type: ignore


class LeftRight(Generic[T]):
    """Wrapper around measurements taken on both the left and right sides.
    This augments the two measurements by adding a meta-measurement with the "avg"/"best" combination of both."""

    __slots__ = ['left', 'right']

    def __init__(self, left: Optional[T], right: Optional[T]):
        if type(left) != type(right) and left is not None and right is not None:  # pylint: disable=C0123
            raise ValueError(f'Left and right types not the same: left="{type(left)}", left="{type(right)}"')
        self.left: Optional[T] = left
        self.right: Optional[T] = right

    def avg(self) -> Optional[T]:
        """meta-measurement with the "avg"/"best" combination of both."""
        return best_effort_avg(self.left, self.right)

    def __eq__(self, other):
        if other is None:
            return False
        if type(other) != type(self):  # pylint: disable=C0123
            raise NotImplementedError
        return ((self.left, self.right) == (other.left, other.right))  # pylint: disable=C0325

    def __repr__(self):
        return f'{self.__class__.__name__}({self.left}, {self.right})'

    def __str__(self):
        return f'left: "{self.left}", right: "{self.right}")'


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import unittest


from .sex import Sex


class SexTest(unittest.TestCase):
    def test_order(self):
        self.assertEqual(sorted([Sex.MALE, Sex.FEMALE]), [Sex.FEMALE, Sex.MALE])
        self.assertEqual(sorted([Sex.MALE_LIKELY, Sex.MALE_ASSUMED]), [Sex.MALE_ASSUMED, Sex.MALE_LIKELY])
        # None is sorted first
        self.assertEqual(sorted([Sex.MALE_LIKELY, Sex.MALE_ASSUMED, None]), [None, Sex.MALE_ASSUMED, Sex.MALE_LIKELY])
        # Numbers are sorted as if they where Sex.value
        self.assertEqual(sorted([0, 100, Sex.MALE_ASSUMED]), [0, Sex.MALE_ASSUMED, 100])
        # Invalid numbers fail
        with self.assertRaises(ValueError):
            [Sex.MALE_ASSUMED, -1].sort()

    def test_parse(self):
        self.assertEqual(Sex.parse('M'), Sex.MALE)
        self.assertEqual(Sex.parse('M?'), Sex.MALE_LIKELY)
        self.assertEqual(Sex.parse('?M'), Sex.MALE_LIKELY)
        self.assertEqual(Sex.parse('M??'), Sex.MALE_ASSUMED)
        self.assertEqual(Sex.parse('??M'), Sex.MALE_ASSUMED)
        self.assertEqual(Sex.parse('?'), Sex.UNKNOWN)
        self.assertEqual(Sex.parse('F??'), Sex.FEMALE_ASSUMED)
        self.assertEqual(Sex.parse('??F'), Sex.FEMALE_ASSUMED)
        self.assertEqual(Sex.parse('F?'), Sex.FEMALE_LIKELY)
        self.assertEqual(Sex.parse('?F'), Sex.FEMALE_LIKELY)
        self.assertEqual(Sex.parse('F'), Sex.FEMALE)

        self.assertEqual(Sex.parse('M'), Sex.parse('m'))
        self.assertEqual(Sex.parse(None), None)
        self.assertEqual(Sex.parse(Sex.MALE), Sex.MALE)


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


from importlib.resources import open_binary
import json
import unittest


from pandas.api.types import CategoricalDtype


from . import test as bioarch_test
from .age import EstimatedAge
from .context import BodyPosition, CompassBearing, Context
from .individual import AgeSexStature, BurialInfo, Individual, LongBoneMeasurement, OsteologicalSex
from .joints import Joints
from .left_right import LeftRight
from .mouth import Mouth
from .occupational_markers import OccupationalMarkers
from .sex import Sex
from .trauma import Trauma


class OsteologicalSexTest(unittest.TestCase):
    def test_to_pd_series(self):
        pelvic = Sex.MALE
        cranium = None
        combined = Sex.MALE_ASSUMED
        os = OsteologicalSex(pelvic, cranium, combined)

        df = os.to_pd_data_frame('id1')

        self.assertIsInstance(df['pelvic_cat'].dtypes, CategoricalDtype)
        self.assertIsInstance(df['pelvic_bin_cat'].dtypes, CategoricalDtype)
        self.assertEqual(df.to_json(orient='records'), '[{"pelvic_cat":"MALE","pelvic_val":100,"pelvic_bin_cat":"MALE","pelvic_bin_val":100,"cranium_cat":null,"cranium_val":null,"cranium_bin_cat":null,"cranium_bin_val":null,"combined_cat":"MALE_ASSUMED","combined_val":80,"combined_bin_cat":"MALE","combined_bin_val":100}]')


class AgeSexStatureTest(unittest.TestCase):
    def test_to_pd_series(self):
        os = OsteologicalSex(Sex.MALE, None, Sex.MALE_ASSUMED)
        age = EstimatedAge.empty()
        femur = LongBoneMeasurement.empty_lr()
        humerus = LongBoneMeasurement.empty_lr()
        tibia = LongBoneMeasurement.empty_lr()
        stature = ''
        body_mass = 'None'

        ass = AgeSexStature(os, age, femur, humerus, tibia, stature, body_mass)

        df = ass.to_pd_data_frame('id1', prefix='ass_')

        with open_binary(bioarch_test, 'AgeSexStatureTest.test_to_pd_series.json') as json_stream:
            expected_json = json.load(json_stream)

        actual_json = json.loads(df.to_json(orient='records'))

        self.assertEqual(expected_json, actual_json)


class LongBoneMeasurementTest(unittest.TestCase):
    def test_lr_avg(self):
        left_femur = LongBoneMeasurement(1.0, None, 1.0, None)
        right_femur = LongBoneMeasurement(2.0, 2.0, None, None)

        femur = LeftRight(left_femur, right_femur)
        avg_femur = femur.avg()
        self.assertEqual(avg_femur.max, 1.5)
        self.assertEqual(avg_femur.bi, 2.0)
        self.assertEqual(avg_femur.head, 1.0)
        self.assertEqual(avg_femur.distal, None)

        femur = LeftRight(LongBoneMeasurement.empty(), right_femur)
        avg_femur = femur.avg()
        self.assertEqual(avg_femur, right_femur)

        femur = LeftRight(left_femur, LongBoneMeasurement.empty())
        avg_femur = femur.avg()
        self.assertEqual(avg_femur, left_femur)

        femur = LeftRight(LongBoneMeasurement.empty(), LongBoneMeasurement.empty())
        avg_femur = femur.avg()
        self.assertEqual(avg_femur, LongBoneMeasurement.empty())


class IndividualTest(unittest.TestCase):
    def test_basic(self):
        burial_info = BurialInfo('site_name', 'site_id')
        age_sex_sature = AgeSexStature.empty()
        mouth = Mouth.empty()
        occupational_markers = OccupationalMarkers.empty()
        joints = Joints.empty()
        trauma = Trauma.empty()
        context = Context.empty()

        individual = Individual('id_1', burial_info, age_sex_sature, mouth, occupational_markers, joints, trauma, context)

        self.assertEqual(individual.id, 'id_1')

    def test_to_pd_data_frame(self):
        burial_info = BurialInfo('site_name', 'site_id')
        age_sex_sature = AgeSexStature.empty()
        mouth = Mouth.empty()
        occupational_markers = OccupationalMarkers.empty()
        joints = Joints.empty()
        trauma = Trauma.empty()
        context = Context(BodyPosition.SUPINE, CompassBearing.WEST, {'spear': True, 'pot': False})

        individual = Individual('id_1', burial_info, age_sex_sature, mouth, occupational_markers, joints, trauma, context)
        df = individual.to_pd_data_frame()

        with open_binary(bioarch_test, 'IndividualTest.test_to_pd_data_frame.json') as json_stream:
            expected_json = json.load(json_stream)

        actual_json = json.loads(df.to_json(orient='records'))

        self.assertEqual(actual_json, expected_json)


def main():
    unittest.main()


if __name__ == "__main__":
    main()



#!/usr/bin/env python


import enum
from enum import Enum
import functools
import logging
from statistics import mean


import pandas as pd


from .left_right import LeftRight


logger = logging.getLogger(__name__)


def is_none_or_na(condition):
    if condition is None:
        return True
    if isinstance(condition, LeftRight):
        return is_none_or_na(condition.avg())
    return condition == JointCondition.NOT_PRESENT


@functools.total_ordering
@enum.unique
class JointCondition(Enum):
    """
    NA- does not exist
    1 - slight/mild
    2 - medium
    3 - extreme
    4 - fused
    5 - SCHMORALS NODES
    6 - FRACTURE
    """
    NOT_PRESENT     = -1  # noqa: E221,E222
    NORMAL          =  0  # noqa: E221,E222
    MILD            =  1  # noqa: E221,E222
    MEDIUM          =  2  # noqa: E221,E222
    EXTREAM         =  3  # noqa: E221,E222
    FUSED           =  4  # noqa: E221,E222
    SCHMORALS_NODES =  5  # noqa: E221,E222
    FRACTURE        =  6  # noqa: E221,E222

    @staticmethod
    def parse(value):
        if value is None:
            return None
        if type(value) == JointCondition:  # pylint: disable=C0123
            return value
        if isinstance(value, int):
            value = str(value)
        if not isinstance(value, str):
            raise ValueError(f'Failed to parse JointCondition: "{value}"')
        value = value.upper()
        for condition in JointCondition:
            if value == condition.name:
                return condition
            if value == str(condition.value):
                return condition
        if value in ('NA', 'N'):
            return JointCondition.NOT_PRESENT
        logger.error('Failed to parse JointCondition: "%s"', value)
        raise ValueError

    @staticmethod
    def avg(left, right):
        if left in (None, JointCondition.NOT_PRESENT):
            return right
        if right in (None, JointCondition.NOT_PRESENT):
            return left
        return JointCondition(int(mean((left.value, right.value))))

    def __lt__(self, other):
        if other is None:
            return False
        if other == float('inf'):
            return True
        if other == float('-inf'):
            return False
        if isinstance(other, int):
            other = JointCondition(other)
        if type(other) != type(self):  # pylint: disable=C0123
            logger.warning('Attempt to compare: %s (%s) with %s (%s)', self, type(self), other, type(other))
            raise NotImplementedError
        return (self.value < other.value)  # pylint: disable=C0325,W0143

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name


class Joints(object):  # pylint: disable=R0902
    """docstring for Joints"""
    def __init__(self, shoulder: LeftRight[JointCondition], elbow: LeftRight[JointCondition], wrist: LeftRight[JointCondition], hip: LeftRight[JointCondition], knee: LeftRight[JointCondition], ankle: LeftRight[JointCondition], sacro_illiac: JointCondition, c1_3: JointCondition, c4_7: JointCondition, t1_4: JointCondition, t5_8: JointCondition, t9_12: JointCondition, l1_5: JointCondition):
        self.shoulder = shoulder
        self.elbow = elbow
        self.wrist = wrist
        self.hip = hip
        self.knee = knee
        self.ankle = ankle
        self.sacro_illiac = sacro_illiac
        self.c1_3 = c1_3
        self.c4_7 = c4_7
        self.t1_4 = t1_4
        self.t5_8 = t5_8
        self.t9_12 = t9_12
        self.l1_5 = l1_5

    @staticmethod
    def empty():
        args = [LeftRight(JointCondition.NOT_PRESENT, JointCondition.NOT_PRESENT)] * 6
        args += [JointCondition.NOT_PRESENT] * 7
        return Joints(*args)

    @staticmethod
    def _to_pd_value(condition: JointCondition):
        if condition is None:
            return None
        if condition == JointCondition.NOT_PRESENT:
            return None
        return condition.name

    def _to_pd_series(self, subset_of_self, prefix):  # pylint: disable=R0201
        labels = []
        values = []
        for key, value in subset_of_self.items():
            if isinstance(value, LeftRight):
                labels.append(f'{prefix}{key}_left')
                values.append(Joints._to_pd_value(value.left))
                labels.append(f'{prefix}{key}_right')
                values.append(Joints._to_pd_value(value.right))
                labels.append(f'{prefix}{key}_avg')
                values.append(Joints._to_pd_value(value.avg()))
            else:
                labels.append(f'{prefix}{key}')
                values.append(Joints._to_pd_value(value))
                labels.append(f'{prefix}{key}_avg')
                values.append(Joints._to_pd_value(value))
        s = pd.Series(values, index=labels, copy=True)

        subset = pd.Series([JointCondition.parse(v).value for k, v in s.items() if not is_none_or_na(v) and k.endswith('_avg')])
        s = s.append(pd.Series([subset.mean(skipna=True)], index=[f'{prefix}mean']))
        s = s.append(pd.Series([subset.max(skipna=True)], index=[f'{prefix}max']))
        s = s.append(pd.Series([subset.min(skipna=True)], index=[f'{prefix}min']))
        s = s.append(pd.Series([subset.count()], index=[f'{prefix}count']))
        return s

    def to_pd_series(self, prefix=''):
        return pd.concat([self._to_pd_series({k: v for k, v in self.__dict__.items()}, f'{prefix}all_'),  # pylint: disable=R1721
                          self._to_pd_series({k: v for k, v in self.__dict__.items() if k.startswith('c')}, f'{prefix}cervical_'),
                          self._to_pd_series({k: v for k, v in self.__dict__.items() if k.startswith('t')}, f'{prefix}thoracic_'),
                          self._to_pd_series({k: v for k, v in self.__dict__.items() if k.startswith('l')}, f'{prefix}lumbar_')])


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import enum
from enum import Enum
import logging


import pandas as pd
from pandas.api.types import CategoricalDtype


from .left_right import LeftRight


logger = logging.getLogger(__name__)


@enum.unique
class TraumaCategory(Enum):
    NOT_PRESENT               = -1    # noqa: E221,E222
    PARTIAL_BONE              =  0.5  # noqa: E221,E222
    NORMAL                    =  1    # noqa: E221,E222
    INFECTION                 =  2    # noqa: E221,E222
    FRACTURE                  =  3    # noqa: E221,E222
    UNHEALED_FRACTURE         =  4    # noqa: E221,E222
    CRIBA                     =  5    # noqa: E221,E222
    BLUNT_FORCE_TRAUMA        =  6    # noqa: E221,E222
    SHARP_FORCE_TRAUMA        =  7    # noqa: E221,E222
    TREPONATION               =  8    # noqa: E221,E222
    UNFUSED                   =  9    # noqa: E221,E222
    BONY_GROWTH               = 10    # noqa: E221,E222
    FUSED                     = 11    # noqa: E221,E222
    OSTEOCHONDRITIS_DESSICANS = 12    # noqa: E221,E222

    @staticmethod
    def parse(value):
        if value is None:
            return None
        if type(value) == TraumaCategory:  # pylint: disable=C0123
            return value
        if isinstance(value, (float, int)):
            value = str(value)
        if not isinstance(value, str):
            raise ValueError(f'Failed to parse TraumaCategory: "{value}"')
        value = value.upper()
        for condition in TraumaCategory:
            if value == condition.name:
                return condition
            if value == str(condition.value):
                return condition
        if value in ('NA', 'N'):
            return TraumaCategory.NOT_PRESENT
        raise ValueError(f'Failed to parse TraumaCategory: "{value}"')

    @staticmethod
    def avg(left, right):
        if left in (None, TraumaCategory.NOT_PRESENT):
            return right
        if right in (None, TraumaCategory.NOT_PRESENT):
            return left
        if left == right:
            return left
        raise NotImplementedError()

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return self.name

    @staticmethod
    def dtype():
        return CategoricalDtype(categories=[s.name for s in TraumaCategory], ordered=False)


class Trauma(object):  # pylint: disable=R0902
    """docstring for Trauma"""

    def __init__(self, facial_bones: TraumaCategory, clavicle: LeftRight[TraumaCategory], scapula: LeftRight[TraumaCategory], humerus: LeftRight[TraumaCategory], ulna: LeftRight[TraumaCategory], radius: LeftRight[TraumaCategory], femur: LeftRight[TraumaCategory], tibia: LeftRight[TraumaCategory], fibula: LeftRight[TraumaCategory], ribs: TraumaCategory, vertabrae: TraumaCategory):
        self.facial_bones = facial_bones

        self.clavicle = clavicle
        self.scapula = scapula
        self.humerus = humerus
        self.ulna = ulna
        self.radius = radius
        self.femur = femur
        self.tibia = tibia
        self.fibula = fibula

        self.ribs = ribs
        self.vertabrae = vertabrae

    @staticmethod
    def empty():
        categories = [TraumaCategory.NOT_PRESENT] * 1
        categories += [LeftRight(TraumaCategory.NOT_PRESENT, TraumaCategory.NOT_PRESENT)] * 8
        categories += [TraumaCategory.NOT_PRESENT] * 2
        return Trauma(*categories)

    def to_pd_data_frame(self, index):
        d = {
            'id': pd.Series([index]),
        }
        for l in ('clavicle', 'scapula', 'humerus', 'ulna', 'radius', 'femur', 'tibia', 'fibula'):
            val = getattr(self, l)
            if val is None:
                continue
            if val.left is not None:
                d[f'{l}_left_cat'] = pd.Series([val.left.name],  copy=True, dtype=TraumaCategory.dtype())  # noqa: E241
                d[f'{l}_left_val'] = pd.Series([val.left.value], copy=True)
            if val.right is not None:
                d[f'{l}_right_cat'] = pd.Series([val.right.name],  copy=True, dtype=TraumaCategory.dtype())  # noqa: E241
                d[f'{l}_right_val'] = pd.Series([val.right.value], copy=True)
            try:
                val_avg = val.avg()
                if val_avg is not None:
                    d[f'{l}_avg_cat'] = pd.Series([val_avg.name],  copy=True, dtype=TraumaCategory.dtype())  # noqa: E241
                    d[f'{l}_avg_val'] = pd.Series([val_avg.value], copy=True)
            except NotImplementedError:
                logger.info('Can not "avg" "%s": "%s"', l, self)

        for l in ('facial_bones', 'ribs', 'vertabrae'):
            val = getattr(self, l)
            if val is None:
                continue
            d[f'{l}_cat'] = pd.Series([val.name],  copy=True, dtype=TraumaCategory.dtype())  # noqa: E241
            d[f'{l}_val'] = pd.Series([val.value], copy=True)

        return pd.DataFrame.from_dict(d).set_index('id')

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        return f'facial_bones="{self.facial_bones}" clavicle="{self.clavicle}" ...'


if __name__ == "__main__":
    raise RuntimeError('No main available')



#!/usr/bin/env python


import logging
from typing import Any, List, Union


import pandas as pd


from .left_right import LeftRight, Optional


logger = logging.getLogger(__name__)


class EnthesialMarker(object):
    """docstring for EnthesialMarker"""
    def __init__(self, value: Union[int, float], is_s: bool = False, is_oe: bool = False):
        self.value = float(value)  # 0-3 in 0.5 increments
        self.is_s = is_s  # 0.5-3 in 0.5 increments ie "s0.5"
        self.is_oe = is_oe  # 0.5-3 in 0.5 increments ie "oe0.5"

        if self.is_s and self.is_oe:
            raise ValueError('Can not be both "s" and "oe" EnthesialMarker')
        if self.is_s or self.is_oe:
            if self.value < 0.5 or self.value > 3:
                raise ValueError(f'Invalid EnthesialMarker value, s/oe should be within 0.5 to 3: {self.value}')
        else:
            if self.value < 0 or self.value > 3:
                raise ValueError(f'Invalid EnthesialMarker value, non s/oe should be within 0 to 3: {self.value}')
        if self.value % 0.5 != 0.0:
            raise ValueError(f'Invalid EnthesialMarker value, not a 0.5 increment: {self.value}')

    @staticmethod
    def parse(value: Any) -> Optional['EnthesialMarker']:
        if value is None:
            return None

        original_value = value
        is_s = False
        is_oe = False
        if isinstance(value, str):
            value = value.lower()
            if value.startswith('r'):
                value = value[1:]
            elif value.startswith('s'):
                is_s = True
                value = value[1:]
            elif value.startswith('oe'):  # TODO: double check logic
                is_oe = True
                value = value[2:]
            elif value.startswith('0e'):  # TODO: double check logic
                logger.warning('Bad EnthesialMarker: %s', value)
                is_oe = True
                value = value[2:]
            elif value.startswith('eo'):  # TODO: double check logic
                logger.warning('Bad EnthesialMarker: %s', value)
                is_oe = True
                value = value[2:]
            elif value.startswith('o'):  # TODO: double check logic
                logger.warning('Bad EnthesialMarker: %s', value)
                is_oe = True
                value = value[1:]
            elif value.startswith('e'):  # TODO: double check logic
                logger.warning('Bad EnthesialMarker: %s', value)
                is_oe = True
                value = value[1:]
            try:
                value = float(value)
            except ValueError as e:
                raise ValueError(f'Unknown EnthesialMarker: "{original_value}"') from e

        if isinstance(value, int):
            value = float(value)

        if isinstance(value, float):
            if value > 6.0:
                value -= 6.0
                is_oe = True
            if value > 3.0:
                value -= 3.0
                is_s = True

        if not isinstance(value, float):
            raise ValueError(f'Unknown EnthesialMarker: "{original_value}"')

        return EnthesialMarker(value, is_s=is_s, is_oe=is_oe)

    def as_num(self) -> float:
        val = self.value
        if self.is_oe:
            val += 6.0
        if self.is_s:
            val += 3.0
        return val

    @staticmethod
    def avg(left: Optional['EnthesialMarker'], right: Optional['EnthesialMarker']) -> Optional['EnthesialMarker']:
        return left if right is None else right

    def __eq__(self, other: Any):
        if other is None:
            return False
        if type(other) != type(self):  # pylint: disable=C0123
            raise NotImplementedError
        return ((self.value, self.is_s, self.is_oe) == (other.value, other.is_s, other.is_oe))  # pylint: disable=C0325

    def __repr__(self):
        return f'{self.__class__.__name__}: {self}'

    def __str__(self):
        s = 's' if self.is_s else ''
        oe = 'oe' if self.is_oe else ''
        return f'{s}{oe}{self.value}'


class OccupationalMarkers(object):  # pylint: disable=R0902
    """docstring for OccupationalMarkers"""
    def __init__(self, c_trapezius: LeftRight[EnthesialMarker], c_o_deltiod: LeftRight[EnthesialMarker], c_o_pectoralis_major: LeftRight[EnthesialMarker], c_costoclaviclar_lig: LeftRight[EnthesialMarker], c_subcalvius: LeftRight[EnthesialMarker], c_conoid_lig: LeftRight[EnthesialMarker], c_trapezoid_lig: LeftRight[EnthesialMarker], s_pectoralis_minor: LeftRight[EnthesialMarker], s_levator_scapulae: LeftRight[EnthesialMarker], s_triceps_long_head: LeftRight[EnthesialMarker], s_trapezius: LeftRight[EnthesialMarker], h_subscapularis: LeftRight[EnthesialMarker], h_teres_major: LeftRight[EnthesialMarker], h_latissimus_dorsi: LeftRight[EnthesialMarker], h_pectoralis_major: LeftRight[EnthesialMarker], h_deltoid: LeftRight[EnthesialMarker], h_coracobrachialis: LeftRight[EnthesialMarker], h_supraspinatus: LeftRight[EnthesialMarker], h_infraspinatus: LeftRight[EnthesialMarker], h_teres_minor: LeftRight[EnthesialMarker], h_o_extensor: LeftRight[EnthesialMarker], h_o_flexor: LeftRight[EnthesialMarker], u_brachialis: LeftRight[EnthesialMarker], u_o_pronator_quadrataus: LeftRight[EnthesialMarker], u_triceps_brachii: LeftRight[EnthesialMarker], u_anconeus: LeftRight[EnthesialMarker], u_o_supinator: LeftRight[EnthesialMarker], r_biceps_brachii: LeftRight[EnthesialMarker], r_supinator: LeftRight[EnthesialMarker], r_pronator_teres: LeftRight[EnthesialMarker], r_pronator_quadratus: LeftRight[EnthesialMarker], r_brachoradialis: LeftRight[EnthesialMarker], f_gluteus_minimus: LeftRight[EnthesialMarker], f_gluteus_medius: LeftRight[EnthesialMarker], f_piriformus: LeftRight[EnthesialMarker], f_obturator_internus: LeftRight[EnthesialMarker], f_obturator_externus: LeftRight[EnthesialMarker], f_quadratis_femoris: LeftRight[EnthesialMarker], f_ilioposas: LeftRight[EnthesialMarker], f_gluteus_maximus: LeftRight[EnthesialMarker], f_pectineus: LeftRight[EnthesialMarker], f_o_vastus_medialis: LeftRight[EnthesialMarker], f_o_vastus_lateralis: LeftRight[EnthesialMarker], f_adductor_magnus: LeftRight[EnthesialMarker], f_o_gastrocnemius: LeftRight[EnthesialMarker], f_o_plantaris: LeftRight[EnthesialMarker], f_o_popliteus: LeftRight[EnthesialMarker], t_tensor_fascia_latae: LeftRight[EnthesialMarker], t_quadriceps: LeftRight[EnthesialMarker], t_sartorius: LeftRight[EnthesialMarker], t_gracilis: LeftRight[EnthesialMarker], t_semitendinosus: LeftRight[EnthesialMarker], t_o_tibialus_anterior: LeftRight[EnthesialMarker], t_biceps_femoris: LeftRight[EnthesialMarker], t_semimembranosus: LeftRight[EnthesialMarker], t_popliteus: LeftRight[EnthesialMarker], t_o_soleus: LeftRight[EnthesialMarker], t_o_tibialis_posterior: LeftRight[EnthesialMarker], t_o_flexor_digitorium: LeftRight[EnthesialMarker], f_biceps_femoris: LeftRight[EnthesialMarker], f_o_extensor_muscles: LeftRight[EnthesialMarker], f_o_flexor_muscles: LeftRight[EnthesialMarker], f_o_peroneus_longus: LeftRight[EnthesialMarker], f_o_peronus_brevis: LeftRight[EnthesialMarker], f_o_soleus: LeftRight[EnthesialMarker], p_quadriceps: LeftRight[EnthesialMarker], c_achilles: LeftRight[EnthesialMarker]):
        self.c_trapezius = c_trapezius
        self.c_o_deltiod = c_o_deltiod
        self.c_o_pectoralis_major = c_o_pectoralis_major
        self.c_costoclaviclar_lig = c_costoclaviclar_lig
        self.c_subcalvius = c_subcalvius
        self.c_conoid_lig = c_conoid_lig
        self.c_trapezoid_lig = c_trapezoid_lig
        self.s_pectoralis_minor = s_pectoralis_minor
        self.s_levator_scapulae = s_levator_scapulae
        self.s_triceps_long_head = s_triceps_long_head
        self.s_trapezius = s_trapezius
        self.h_subscapularis = h_subscapularis
        self.h_teres_major = h_teres_major
        self.h_latissimus_dorsi = h_latissimus_dorsi
        self.h_pectoralis_major = h_pectoralis_major
        self.h_deltoid = h_deltoid
        self.h_coracobrachialis = h_coracobrachialis
        self.h_supraspinatus = h_supraspinatus
        self.h_infraspinatus = h_infraspinatus
        self.h_teres_minor = h_teres_minor
        self.h_o_extensor = h_o_extensor
        self.h_o_flexor = h_o_flexor
        self.u_brachialis = u_brachialis
        self.u_o_pronator_quadrataus = u_o_pronator_quadrataus
        self.u_triceps_brachii = u_triceps_brachii
        self.u_anconeus = u_anconeus
        self.u_o_supinator = u_o_supinator
        self.r_biceps_brachii = r_biceps_brachii
        self.r_supinator = r_supinator
        self.r_pronator_teres = r_pronator_teres
        self.r_pronator_quadratus = r_pronator_quadratus
        self.r_brachoradialis = r_brachoradialis
        self.f_gluteus_minimus = f_gluteus_minimus
        self.f_gluteus_medius = f_gluteus_medius
        self.f_piriformus = f_piriformus
        self.f_obturator_internus = f_obturator_internus
        self.f_obturator_externus = f_obturator_externus
        self.f_quadratis_femoris = f_quadratis_femoris
        self.f_ilioposas = f_ilioposas
        self.f_gluteus_maximus = f_gluteus_maximus
        self.f_pectineus = f_pectineus
        self.f_o_vastus_medialis = f_o_vastus_medialis
        self.f_o_vastus_lateralis = f_o_vastus_lateralis
        self.f_adductor_magnus = f_adductor_magnus
        self.f_o_gastrocnemius = f_o_gastrocnemius
        self.f_o_plantaris = f_o_plantaris
        self.f_o_popliteus = f_o_popliteus
        self.t_tensor_fascia_latae = t_tensor_fascia_latae
        self.t_quadriceps = t_quadriceps
        self.t_sartorius = t_sartorius
        self.t_gracilis = t_gracilis
        self.t_semitendinosus = t_semitendinosus
        self.t_o_tibialus_anterior = t_o_tibialus_anterior
        self.t_biceps_femoris = t_biceps_femoris
        self.t_semimembranosus = t_semimembranosus
        self.t_popliteus = t_popliteus
        self.t_o_soleus = t_o_soleus
        self.t_o_tibialis_posterior = t_o_tibialis_posterior
        self.t_o_flexor_digitorium = t_o_flexor_digitorium
        self.f_biceps_femoris = f_biceps_femoris
        self.f_o_extensor_muscles = f_o_extensor_muscles
        self.f_o_flexor_muscles = f_o_flexor_muscles
        self.f_o_peroneus_longus = f_o_peroneus_longus
        self.f_o_peronus_brevis = f_o_peronus_brevis
        self.f_o_soleus = f_o_soleus
        self.p_quadriceps = p_quadriceps
        self.c_achilles = c_achilles

    @staticmethod
    def empty() -> 'OccupationalMarkers':
        markers: List[LeftRight[EnthesialMarker]] = [LeftRight(None, None)] * 67
        return OccupationalMarkers(*markers)

    def to_pd_series(self, prefix: str = '') -> pd.Series:
        labels = []
        values = []
        for key, value in self.__dict__.items():
            labels.append(f'{prefix}{key}_left')
            values.append(value.left)
            labels.append(f'{prefix}{key}_right')
            values.append(value.right)
            labels.append(f'{prefix}{key}_avg')
            values.append(value.avg())
            labels.append(f'{prefix}{key}_avg_val')
            values.append(None if value.avg() is None else value.avg().as_num())
        s = pd.Series(values, index=labels, copy=True)

        subset = pd.Series([v.as_num() for k, v in s.items() if v is not None and k.endswith('_avg')])
        s = s.append(pd.Series([subset.mean(skipna=True)], index=[f'{prefix}mean']))
        s = s.append(pd.Series([subset.max(skipna=True)], index=[f'{prefix}max']))
        s = s.append(pd.Series([subset.min(skipna=True)], index=[f'{prefix}min']))
        s = s.append(pd.Series([subset.count()], index=[f'{prefix}count']))
        return s


if __name__ == "__main__":
    raise RuntimeError('No main available')
